package main

import (
    "net/http"
    "time"
    "strconv"

    "github.com/gin-gonic/gin"
    )

type Task struct {
 ID          string    `json:"id"`
 Title       string    `json:"title"`
 Description string    `json:"description"`
 DueDate     time.Time `json:"due_date"`
 Status      string    `json:"status"`
}


func main() {
    taskmap := make(map[string]Task)

    taskmap["1"] = Task{
        ID: "1",
        Title: "Task 1",
        Description: "First task",
        DueDate: time.Now(),
        Status: "Pending",
    }

    router := gin.Default()

    router.GET("/ping", func(ctx *gin.Context) {
        ctx.JSON(200, gin.H{
            "message": "pong",
        })
    })

    router.GET("/tasks", func(ctx *gin.Context){
        // FIXME: We should append some headers here to avoid this getting cached improperly!

        tasks := make([]Task, 0, len(taskmap))
        for _, t := range taskmap {
            tasks = append(tasks, t)
        }

        ctx.JSON(http.StatusOK, gin.H{
            "tasks": tasks,
        })
    })

    router.GET("/tasks/:id", func(ctx *gin.Context) {
        id := ctx.Param("id")

        if task, ok := taskmap[id]; ok {
            ctx.JSON(http.StatusOK, task)
            return
        }

        ctx.JSON(http.StatusNotFound, gin.H{"error": "Task not found"})
    })

    router.PUT("/tasks/:id", func(ctx *gin.Context) {
        id := ctx.Param("id")

        var updatedTask Task

        if err := ctx.ShouldBindJSON(&updatedTask); err != nil {
            ctx.JSON(http.StatusBadRequest, gin.H{"error": "Task not found"})
            return
        }

        if task, ok := taskmap[id]; ok {
            if updatedTask.Title != "" {
                task.Title = updatedTask.Title
            }

            if updatedTask.Description != "" {
                task.Description = updatedTask.Description
            }

            delete(taskmap, id)
            taskmap[id] = task

            ctx.JSON(http.StatusOK, gin.H{"message": "Task updated"})
            return
        }

        ctx.JSON(http.StatusNotFound, gin.H{"message": "Task not found"})
    })

    router.DELETE("/tasks/:id", func(ctx *gin.Context) {
        id := ctx.Param("id")

        if _, ok := taskmap[id]; ok {
            delete(taskmap, id)
            ctx.JSON(http.StatusOK, gin.H{"message": "Task removed"})
            return
        }

        ctx.JSON(http.StatusNotFound, gin.H {"message": "Task not found"})
    })

    router.POST("/tasks", func(ctx *gin.Context) {
        var newTask Task

        if err := ctx.ShouldBindJSON(&newTask); err != nil {
            ctx.JSON(http.StatusBadRequest, gin.H{"error": err.Error()})
            return
        }

        // generate id if empty
        if newTask.ID == "" {
            for i:=1; ; i++ {
                id := strconv.Itoa(i)
                if _, found := taskmap[id]; !found {
                    newTask.ID = id
                    break
                }
            }
        }

        // reject if id already in use
        if _, found := taskmap[newTask.ID]; found {
            ctx.JSON(http.StatusConflict, gin.H {"error": "ID already allocated"})
            return
        }

        taskmap[newTask.ID] = newTask

        ctx.JSON(http.StatusCreated, gin.H{
            "message": "Task created",
            "id": newTask.ID,
        })
    })

    router.Run()
}
